#!/bin/sh

#
# Quick and dirty shell-script that tests various block-size with
# dd(1) in a loop where for each run the VM gets restarted.
#

STATE_FILE="/rw/state"
LOG_FILE="/rw/log"
LOG_DIR="/rw/logs"
RESULTS_FILE="/rw/results"

SRC_FILE=/mnt/src.img

if [ ! -f "$STATE_FILE" ]; then
	echo initialize > $STATE_FILE
fi

STATE=$(head -1 $STATE_FILE)

execute_dd() {
	src_file="$1"
	dst_file="$2"
	dd_bs="$3"
	cmd="dd if=$src_file of=$dst_file bs=$dd_bs oflag=direct iflag=direct conv=fsync"
	dd_log_file="$LOG_DIR/dd.${dd_bs}"
	echo "START $dd_bs"|tee -a $LOG_FILE
	echo "$cmd"|tee -a $LOG_FILE
	$cmd 2>&1|tee -a $dd_log_file
	cat $dd_log_file >> $LOG_FILE
	echo "END $dd_bs"|tee -a $LOG_FILE
}

read_block_size() {
	NEW_BS=$(head -1 "$1")
	sed -i 1d "$1"
}

read_first_line() {
	LINE="$(head -1 "$1")"
	sed -i 1d "$1"
}

execute_fio() {
	fio_log_file="$LOG_DIR/fio.$1"
	sh /rw/run_fio.sh $* | tee -a $fio_log_file
	cat $fio_log_file >> $LOG_FILE
}

generate_results() {
	for dd_result_file in $LOG_DIR/dd.*; do
		result="$(tail -1 $dd_result_file)"
		dd_bs="${dd_result_file#*.}"
		echo "@@ dd $dd_bs $result" >> $RESULTS_FILE
	done
	for fio_result_file in $LOG_DIR/fio.*; do
		result="$(awk '/: bw=/ { print $0 }' $fio_result_file)"
		fio_test="${fio_result_file#*.}"
		echo "@@ fio $fio_test $result" >> $RESULTS_FILE
	done
}

if [ -f "/rw/fio.tar" ]; then
	tar xf /rw/fio.tar -C .
fi

case "$STATE" in
initialize)
	mkfs.ext2 /dev/sda  || exit 1
	mount /dev/sda /mnt || exit 3
	mkdir -p $LOG_DIR   || exit 4

	dd if=/dev/zero of=$SRC_FILE bs=1M count=1024

# prepare dd
	cat << EOF > /rw/block_sizes
4096
8K
32K
128K
512K
2M
8M
EOF
# prepare fio
cat << EOF > /rw/fio_tests
randread libaio 0 1 4k /mnt/fiofile
randwrite libaio 0 1 4k /mnt/fiofile
read libaio 0 1 4k /mnt/fiofile
write libaio 0 1 4k /mnt/fiofile
EOF
	echo "block_size" > $STATE_FILE
	reboot
	break ;;
block_size)
	mount /dev/sda /mnt || exit 3

	read_block_size "/rw/block_sizes"
	if [ "$NEW_BS" != "" ]; then
		execute_dd "$SRC_FILE" "/mnt/dst.img" "$NEW_BS"
		reboot
	else
		echo "finished dd tests"
		echo "fio" > $STATE_FILE
		reboot
	fi
	break ;;
fio)
	mount /dev/sda /mnt || exit 3
	read_first_line "/rw/fio_tests"
	if [ "$LINE" != "" ]; then
		execute_fio $LINE
		reboot
	else
		echo "finished fio tests"
		echo "finished" > $STATE_FILE
		reboot
	fi
	break ;;
finished)
	generate_results
	cat $RESULTS_FILE
	echo "finished testing"
	break ;;
esac

exit 0
